@@ -339,7 +339,7 @@ nautilus_gtk_sidebar_row_set_property (GObject      *object,
       if (self->section_type == NAUTILUS_GTK_PLACES_SECTION_DEFAULT_LOCATIONS)
         gtk_label_set_ellipsize (GTK_LABEL (self->label_widget), PANGO_ELLIPSIZE_NONE);
       else
-        gtk_label_set_ellipsize (GTK_LABEL (self->label_widget), PANGO_ELLIPSIZE_END);
+        gtk_label_set_ellipsize (GTK_LABEL (self->label_widget), PANGO_ELLIPSIZE_MIDDLE);
       break;
 
     case PROP_PLACE_TYPE:
