@@ -32,6 +32,7 @@
 #include <fcntl.h>
 #include <gdk/gdk.h>
 #include <gio/gio.h>
+#include <gio/gunixmounts.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
@@ -88,6 +89,8 @@ typedef struct
 
     NautilusTagManager *tag_manager;
 
+    GUnixMountMonitor *mount_monitor;
+
     NautilusDBusLauncher *dbus_launcher;
 } NautilusApplicationPrivate;
 
@@ -332,16 +335,14 @@ nautilus_application_open_location_full (NautilusApplication *self,
                                          NautilusOpenFlags    flags,
                                          GList               *selection,
                                          NautilusWindow      *target_window,
-                                         NautilusWindowSlot  *target_slot,
-                                         const char          *startup_id)
+                                         NautilusWindowSlot  *target_slot)
 {
     NAUTILUS_APPLICATION_CLASS (G_OBJECT_GET_CLASS (self))->open_location_full (self,
                                                                                 location,
                                                                                 flags,
                                                                                 selection,
                                                                                 target_window,
-                                                                                target_slot,
-                                                                                startup_id);
+                                                                                target_slot);
 }
 
 static void
@@ -350,8 +351,7 @@ real_open_location_full (NautilusApplication *self,
                          NautilusOpenFlags    flags,
                          GList               *selection,
                          NautilusWindow      *target_window,
-                         NautilusWindowSlot  *target_slot,
-                         const char          *startup_id)
+                         NautilusWindowSlot  *target_slot)
 {
     NautilusWindowSlot *active_slot = NULL;
     NautilusWindow *active_window;
@@ -447,7 +447,7 @@ real_open_location_full (NautilusApplication *self,
     /* Application is the one that manages windows, so this flag shouldn't use
      * it anymore by any client */
     flags &= ~NAUTILUS_OPEN_FLAG_NEW_WINDOW;
-    nautilus_window_open_location_full (target_window, location, flags, selection, target_slot, startup_id);
+    nautilus_window_open_location_full (target_window, location, flags, selection, target_slot);
 }
 
 static NautilusWindow *
@@ -458,13 +458,13 @@ open_window (NautilusApplication *self,
 
     if (location != NULL)
     {
-        nautilus_application_open_location_full (self, location, 0, NULL, window, NULL, NULL);
+        nautilus_application_open_location_full (self, location, 0, NULL, window, NULL);
     }
     else
     {
         GFile *home;
         home = g_file_new_for_path (g_get_home_dir ());
-        nautilus_application_open_location_full (self, home, 0, NULL, window, NULL, NULL);
+        nautilus_application_open_location_full (self, home, 0, NULL, window, NULL);
 
         g_object_unref (home);
     }
@@ -481,12 +481,6 @@ nautilus_application_open_location (NautilusApplication *self,
     NautilusWindow *window;
     NautilusWindowSlot *slot;
     GList *sel_list = NULL;
-    g_autofree char *location_uri = g_file_get_uri (location);
-
-    if (location_uri[0] == '\0')
-    {
-        return;
-    }
 
     if (selection != NULL)
     {
@@ -504,7 +498,7 @@ nautilus_application_open_location (NautilusApplication *self,
         window = get_nautilus_window_containing_slot (slot);
     }
 
-    nautilus_application_open_location_full (self, location, 0, sel_list, window, slot, startup_id);
+    nautilus_application_open_location_full (self, location, 0, sel_list, window, slot);
 
     if (sel_list != NULL)
     {
@@ -549,7 +543,7 @@ nautilus_application_open (GApplication  *app,
         else
         {
             /* We open the location again to update any possible selection */
-            nautilus_application_open_location_full (NAUTILUS_APPLICATION (app), file, 0, NULL, NULL, slot, NULL);
+            nautilus_application_open_location_full (NAUTILUS_APPLICATION (app), file, 0, NULL, NULL, slot);
         }
     }
 }
@@ -576,6 +570,8 @@ nautilus_application_finalize (GObject *object)
 
     g_clear_object (&priv->tag_manager);
 
+    g_clear_object (&priv->mount_monitor);
+
     g_clear_object (&priv->dbus_launcher);
 
     nautilus_trash_monitor_clear ();
@@ -650,7 +646,7 @@ action_new_window (GSimpleAction *action,
 
     nautilus_application_open_location_full (application, home,
                                              NAUTILUS_OPEN_FLAG_NEW_WINDOW,
-                                             NULL, NULL, NULL, NULL);
+                                             NULL, NULL, NULL);
 }
 
 static void
@@ -692,7 +688,7 @@ action_clone_window (GSimpleAction *action,
     }
 
     nautilus_application_open_location_full (NAUTILUS_APPLICATION (application), location,
-                                             NAUTILUS_OPEN_FLAG_NEW_WINDOW, NULL, NULL, NULL, NULL);
+                                             NAUTILUS_OPEN_FLAG_NEW_WINDOW, NULL, NULL, NULL);
 }
 
 static void
@@ -736,7 +732,7 @@ action_help (GSimpleAction *action,
              gpointer       user_data)
 {
     GtkWindow *window;
-    GtkWidget *dialog;
+    AdwDialog *dialog;
     GtkApplication *application = user_data;
     GError *error = NULL;
 
@@ -745,15 +741,11 @@ action_help (GSimpleAction *action,
 
     if (error)
     {
-        dialog = adw_message_dialog_new (window ? GTK_WINDOW (window) : NULL,
-                                         NULL, NULL);
-        adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog),
-                                           _("There was an error displaying help: \n%s"),
-                                           error->message);
-        adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "ok", _("_OK"));
-        adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "ok");
+        dialog = adw_alert_dialog_new (_("There was an error displaying help"), error->message);
+        adw_alert_dialog_add_response (ADW_ALERT_DIALOG (dialog), "ok", _("_OK"));
+        adw_alert_dialog_set_default_response (ADW_ALERT_DIALOG (dialog), "ok");
 
-        gtk_window_present (GTK_WINDOW (dialog));
+        adw_dialog_present (dialog, GTK_WIDGET (window));
         g_error_free (error);
     }
 }
@@ -1009,6 +1001,11 @@ nautilus_application_init (NautilusApplication *self)
     priv->undo_manager = nautilus_file_undo_manager_new ();
     priv->tag_manager = nautilus_tag_manager_new ();
 
+    /* Retain a mount monitor so GIO's caching works. This helps to speed
+     * up various filesystem queries, e.g. those determining whether to
+     * recurse during searches. */
+    priv->mount_monitor = g_unix_mount_monitor_get ();
+
     priv->dbus_launcher = nautilus_dbus_launcher_new ();
 
     nautilus_tracker_setup_miner_fs_connection ();
@@ -1424,7 +1421,7 @@ nautilus_application_window_removed (GtkApplication *app,
     }
 
     /* if this was the last window, close the previewer */
-    if (g_list_length (priv->windows) == 0)
+    if (priv->windows == NULL)
     {
         nautilus_previewer_call_close ();
         nautilus_progress_persistence_handler_make_persistent (priv->progress_handler);
