@@ -75,14 +75,18 @@ static void nautilus_bookmark_disconnect_file (NautilusBookmark *file);
 
 G_DEFINE_TYPE (NautilusBookmark, nautilus_bookmark, G_TYPE_OBJECT);
 
-static void
-nautilus_bookmark_set_name_internal (NautilusBookmark *bookmark,
-                                     const char       *new_name)
+void
+nautilus_bookmark_set_name (NautilusBookmark *bookmark,
+                            const char       *new_name)
 {
-    if (g_strcmp0 (bookmark->name, new_name) != 0)
+    if (g_set_str (&bookmark->name, new_name))
     {
-        g_free (bookmark->name);
-        bookmark->name = g_strdup (new_name);
+        if ((new_name == NULL && bookmark->has_custom_name) ||
+            (new_name != NULL && !bookmark->has_custom_name))
+        {
+            bookmark->has_custom_name = !bookmark->has_custom_name;
+            g_object_notify_by_pspec (G_OBJECT (bookmark), properties[PROP_CUSTOM_NAME]);
+        }
 
         g_object_notify_by_pspec (G_OBJECT (bookmark), properties[PROP_NAME]);
     }
@@ -101,17 +105,13 @@ bookmark_set_name_from_ready_file (NautilusBookmark *self,
 
     display_name = nautilus_file_get_display_name (self->file);
 
-    if (nautilus_file_is_other_locations (self->file))
+    if (nautilus_file_is_home (self->file))
     {
-        nautilus_bookmark_set_name_internal (self, _("Other Locations"));
-    }
-    else if (nautilus_file_is_home (self->file))
-    {
-        nautilus_bookmark_set_name_internal (self, _("Home"));
+        nautilus_bookmark_set_name (self, _("Home"));
     }
     else if (g_strcmp0 (self->name, display_name) != 0)
     {
-        nautilus_bookmark_set_name_internal (self, display_name);
+        nautilus_bookmark_set_name (self, display_name);
         g_debug ("%s: name changed to %s", nautilus_bookmark_get_name (self), display_name);
     }
 }
@@ -165,103 +165,59 @@ bookmark_file_changed_callback (NautilusFile     *file,
     }
 }
 
-gboolean
-nautilus_bookmark_get_is_builtin (NautilusBookmark *bookmark)
-{
-    GUserDirectory xdg_type;
-
-    /* if this is not an XDG dir, it's never builtin */
-    if (!nautilus_bookmark_get_xdg_type (bookmark, &xdg_type))
-    {
-        return FALSE;
-    }
-
-    /* exclude XDG locations which are not in our builtin list */
-    return (xdg_type != G_USER_DIRECTORY_DESKTOP) &&
-           (xdg_type != G_USER_DIRECTORY_TEMPLATES) &&
-           (xdg_type != G_USER_DIRECTORY_PUBLIC_SHARE);
-}
-
-gboolean
+static gboolean
 nautilus_bookmark_get_xdg_type (NautilusBookmark *bookmark,
                                 GUserDirectory   *directory)
 {
-    gboolean match;
-    GFile *location;
-    const gchar *path;
-    GUserDirectory dir;
-
-    match = FALSE;
-
-    for (dir = 0; dir < G_USER_N_DIRECTORIES; dir++)
+    for (GUserDirectory dir = 0; dir < G_USER_N_DIRECTORIES; dir++)
     {
-        path = g_get_user_special_dir (dir);
-        if (!path)
+        const gchar *path = g_get_user_special_dir (dir);
+
+        if (path == NULL)
         {
             continue;
         }
 
-        location = g_file_new_for_path (path);
-        match = g_file_equal (location, bookmark->location);
-        g_object_unref (location);
+        g_autoptr (GFile) location = g_file_new_for_path (path);
 
-        if (match)
+        if (g_file_equal (location, bookmark->location))
         {
-            break;
+            if (directory != NULL)
+            {
+                *directory = dir;
+            }
+            return TRUE;
         }
     }
 
-    if (match && directory != NULL)
-    {
-        *directory = dir;
-    }
-
-    return match;
+    return FALSE;
 }
 
 static GIcon *
 get_native_icon (NautilusBookmark *bookmark,
                  gboolean          symbolic)
 {
+    if (!bookmark->exists)
+    {
+        g_debug ("%s: file does not exist, set warning icon", nautilus_bookmark_get_name (bookmark));
+        return g_themed_icon_new (symbolic ? "dialog-warning-symbolic" : "dialog-warning");
+    }
+
     GUserDirectory xdg_type;
-    GIcon *icon = NULL;
 
-    if (bookmark->file == NULL)
-    {
-        goto out;
-    }
-
-    if (!nautilus_bookmark_get_xdg_type (bookmark, &xdg_type))
-    {
-        goto out;
-    }
-
-    if (xdg_type < G_USER_N_DIRECTORIES)
+    if (nautilus_bookmark_get_xdg_type (bookmark, &xdg_type))
     {
         if (symbolic)
         {
-            icon = nautilus_special_directory_get_symbolic_icon (xdg_type);
+            return nautilus_special_directory_get_symbolic_icon (xdg_type);
         }
         else
         {
-            icon = nautilus_special_directory_get_icon (xdg_type);
+            return nautilus_special_directory_get_icon (xdg_type);
         }
     }
 
-out:
-    if (icon == NULL)
-    {
-        if (symbolic)
-        {
-            icon = g_themed_icon_new (NAUTILUS_ICON_FOLDER);
-        }
-        else
-        {
-            icon = g_themed_icon_new (NAUTILUS_ICON_FULLCOLOR_FOLDER);
-        }
-    }
-
-    return icon;
+    return g_themed_icon_new (symbolic ? NAUTILUS_ICON_FOLDER : NAUTILUS_ICON_FULLCOLOR_FOLDER);
 }
 
 static void
@@ -270,13 +226,7 @@ nautilus_bookmark_set_icon_to_default (NautilusBookmark *bookmark)
     g_autoptr (GIcon) icon = NULL;
     g_autoptr (GIcon) symbolic_icon = NULL;
 
-    if (!bookmark->exists)
-    {
-        g_debug ("%s: file does not exist, set warning icon", nautilus_bookmark_get_name (bookmark));
-        symbolic_icon = g_themed_icon_new ("dialog-warning-symbolic");
-        icon = g_themed_icon_new ("dialog-warning");
-    }
-    else if (g_file_is_native (bookmark->location))
+    if (g_file_is_native (bookmark->location))
     {
         symbolic_icon = get_native_icon (bookmark, TRUE);
         icon = get_native_icon (bookmark, FALSE);
@@ -492,7 +442,7 @@ nautilus_bookmark_set_property (GObject      *object,
 
         case PROP_NAME:
         {
-            nautilus_bookmark_set_name_internal (self, g_value_get_string (value));
+            nautilus_bookmark_set_name (self, g_value_get_string (value));
         }
         break;
 
@@ -606,7 +556,7 @@ nautilus_bookmark_class_init (NautilusBookmarkClass *class)
                              "Bookmark's name",
                              "The name of this bookmark",
                              NULL,
-                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
+                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY);
 
     properties[PROP_CUSTOM_NAME] =
         g_param_spec_boolean ("custom-name",
