@@ -40,79 +40,47 @@ typedef struct
     GFile *to;
 } NautilusFileChange;
 
-typedef struct
-{
-    GList *head;
-    GList *tail;
-    GMutex mutex;
-} NautilusFileChangesQueue;
-
-static NautilusFileChangesQueue *
-nautilus_file_changes_queue_new (void)
-{
-    NautilusFileChangesQueue *result;
-
-    result = g_new0 (NautilusFileChangesQueue, 1);
-    g_mutex_init (&result->mutex);
-
-    return result;
-}
-
-static NautilusFileChangesQueue *
+static GAsyncQueue *
 nautilus_file_changes_queue_get (void)
 {
-    static NautilusFileChangesQueue *file_changes_queue;
+    static GAsyncQueue *file_changes_queue;
+    static gsize init_value = 0;
 
-    if (file_changes_queue == NULL)
+    if (g_once_init_enter (&init_value))
     {
-        file_changes_queue = nautilus_file_changes_queue_new ();
+        file_changes_queue = g_async_queue_new ();
+        g_once_init_leave (&init_value, 1);
     }
 
     return file_changes_queue;
 }
 
-static void
-nautilus_file_changes_queue_add_common (NautilusFileChangesQueue *queue,
-                                        NautilusFileChange       *new_item)
-{
-    /* enqueue the new queue item while locking down the list */
-    g_mutex_lock (&queue->mutex);
-
-    queue->head = g_list_prepend (queue->head, new_item);
-    if (queue->tail == NULL)
-    {
-        queue->tail = queue->head;
-    }
-
-    g_mutex_unlock (&queue->mutex);
-}
-
 void
 nautilus_file_changes_queue_file_added (GFile *location)
 {
     NautilusFileChange *new_item;
-    NautilusFileChangesQueue *queue;
+    GAsyncQueue *queue;
 
     queue = nautilus_file_changes_queue_get ();
 
     new_item = g_new0 (NautilusFileChange, 1);
     new_item->kind = CHANGE_FILE_ADDED;
     new_item->from = g_object_ref (location);
-    nautilus_file_changes_queue_add_common (queue, new_item);
+    g_async_queue_push (queue, new_item);
 }
 
 void
 nautilus_file_changes_queue_file_changed (GFile *location)
 {
     NautilusFileChange *new_item;
-    NautilusFileChangesQueue *queue;
+    GAsyncQueue *queue;
 
     queue = nautilus_file_changes_queue_get ();
 
     new_item = g_new0 (NautilusFileChange, 1);
     new_item->kind = CHANGE_FILE_CHANGED;
     new_item->from = g_object_ref (location);
-    nautilus_file_changes_queue_add_common (queue, new_item);
+    g_async_queue_push (queue, new_item);
 }
 
 /* A specialized variant of nautilus_file_changes_queue_file_removed(). */
@@ -120,28 +88,28 @@ void
 nautilus_file_changes_queue_file_unmounted (GFile *location)
 {
     NautilusFileChange *new_item;
-    NautilusFileChangesQueue *queue;
+    GAsyncQueue *queue;
 
     queue = nautilus_file_changes_queue_get ();
 
     new_item = g_new0 (NautilusFileChange, 1);
     new_item->kind = CHANGE_FILE_UNMOUNTED;
     new_item->from = g_object_ref (location);
-    nautilus_file_changes_queue_add_common (queue, new_item);
+    g_async_queue_push (queue, new_item);
 }
 
 void
 nautilus_file_changes_queue_file_removed (GFile *location)
 {
     NautilusFileChange *new_item;
-    NautilusFileChangesQueue *queue;
+    GAsyncQueue *queue;
 
     queue = nautilus_file_changes_queue_get ();
 
     new_item = g_new0 (NautilusFileChange, 1);
     new_item->kind = CHANGE_FILE_REMOVED;
     new_item->from = g_object_ref (location);
-    nautilus_file_changes_queue_add_common (queue, new_item);
+    g_async_queue_push (queue, new_item);
 }
 
 void
@@ -149,7 +117,7 @@ nautilus_file_changes_queue_file_moved (GFile *from,
                                         GFile *to)
 {
     NautilusFileChange *new_item;
-    NautilusFileChangesQueue *queue;
+    GAsyncQueue *queue;
 
     queue = nautilus_file_changes_queue_get ();
 
@@ -157,37 +125,7 @@ nautilus_file_changes_queue_file_moved (GFile *from,
     new_item->kind = CHANGE_FILE_MOVED;
     new_item->from = g_object_ref (from);
     new_item->to = g_object_ref (to);
-    nautilus_file_changes_queue_add_common (queue, new_item);
-}
-
-static NautilusFileChange *
-nautilus_file_changes_queue_get_change (NautilusFileChangesQueue *queue)
-{
-    GList *new_tail;
-    NautilusFileChange *result;
-
-    g_assert (queue != NULL);
-
-    /* dequeue the tail item while locking down the list */
-    g_mutex_lock (&queue->mutex);
-
-    if (queue->tail == NULL)
-    {
-        result = NULL;
-    }
-    else
-    {
-        new_tail = queue->tail->prev;
-        result = queue->tail->data;
-        queue->head = g_list_remove_link (queue->head,
-                                          queue->tail);
-        g_list_free_1 (queue->tail);
-        queue->tail = new_tail;
-    }
-
-    g_mutex_unlock (&queue->mutex);
-
-    return result;
+    g_async_queue_push (queue, new_item);
 }
 
 static void
@@ -220,7 +158,7 @@ nautilus_file_changes_consume_changes (void)
     GList *additions, *changes, *deletions, *moves;
     GList *unmounts = NULL;
     GFilePair *pair;
-    NautilusFileChangesQueue *queue;
+    GAsyncQueue *queue;
     gboolean flush_needed;
 
 
@@ -238,7 +176,7 @@ nautilus_file_changes_consume_changes (void)
      */
     for (;;)
     {
-        change = nautilus_file_changes_queue_get_change (queue);
+        change = g_async_queue_try_pop (queue);
 
         /* figure out if we need to flush the pending changes that we collected sofar */
 
