@@ -49,8 +49,6 @@ typedef void (* NautilusUnmountCallback)   (gpointer    callback_data);
 typedef void (* NautilusExtractCallback)   (GList    *outputs,
                                             gpointer  callback_data);
 
-/* FIXME: int copy_action should be an enum */
-
 void nautilus_file_operations_copy_move   (const GList                    *item_uris,
                                            const char                     *target_dir_uri,
                                            GdkDragAction                   copy_action,
@@ -70,7 +68,7 @@ void nautilus_file_operations_new_folder  (GtkWidget                      *paren
 void nautilus_file_operations_new_file    (GtkWidget                 *parent_view,
 					   const char                *parent_dir,
 					   const char                *target_filename,
-					   const char                *initial_contents,
+					   const void                *initial_contents,
 					   gsize                      length,
 					   NautilusCreateCallback     done_callback,
 					   gpointer                   data);
