@@ -23,13 +23,13 @@
 #include "nautilus-file-utilities.h"
 
 #include "nautilus-application.h"
+#include "nautilus-file.h"
+#include "nautilus-file-operations.h"
 #include "nautilus-filename-utilities.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-icon-names.h"
 #include "nautilus-metadata.h"
-#include "nautilus-file.h"
-#include "nautilus-file-operations.h"
-#include "nautilus-filename-utilities.h"
+#include "nautilus-network-directory.h"
 #include "nautilus-scheme.h"
 #include "nautilus-search-directory.h"
 #include "nautilus-starred-directory.h"
@@ -70,18 +70,8 @@ nautilus_compute_title_for_location (GFile *location)
     {
         file = nautilus_file_get (location);
 
-        if (nautilus_file_is_other_locations (file))
-        {
-            title = g_strdup (_("Other Locations"));
-        }
-        else if (nautilus_file_is_starred_location (file))
-        {
-            title = g_strdup (_("Starred"));
-        }
-        else
-        {
-            title = g_strdup (nautilus_file_get_display_name (file));
-        }
+        title = g_strdup (nautilus_file_get_display_name (file));
+
         nautilus_file_unref (file);
     }
 
@@ -135,7 +125,16 @@ nautilus_get_user_directory (void)
 char *
 nautilus_get_scripts_directory_path (void)
 {
-    return g_build_filename (g_get_user_data_dir (), "nautilus", "scripts", NULL);
+    if (nautilus_application_is_sandboxed ())
+    {
+        /* g_get_user_data_dir() leads to a different path then expected under
+         * the flatpak sandbox */
+        return g_build_filename (g_get_home_dir (), ".local", "share", "nautilus", "scripts", NULL);
+    }
+    else
+    {
+        return g_build_filename (g_get_user_data_dir (), "nautilus", "scripts", NULL);
+    }
 }
 
 char *
@@ -265,7 +264,8 @@ nautilus_get_mounted_mount_for_root (GFile *location)
 
 GFile *
 nautilus_generate_unique_file_in_directory (GFile      *directory,
-                                            const char *basename)
+                                            const char *basename,
+                                            gboolean    ignore_extension)
 {
     g_return_val_if_fail (directory != NULL, NULL);
     g_return_val_if_fail (basename != NULL, NULL);
@@ -275,7 +275,10 @@ nautilus_generate_unique_file_in_directory (GFile      *directory,
 
     for (size_t counter = 1; g_file_query_exists (child, NULL); counter += 1)
     {
-        g_autofree char *filename = nautilus_filename_for_conflict (basename, counter, -1, FALSE);
+        g_autofree char *filename = nautilus_filename_for_conflict (basename,
+                                                                    counter,
+                                                                    -1,
+                                                                    ignore_extension);
 
         g_object_unref (child);
         child = g_file_get_child (directory, filename);
@@ -672,37 +675,27 @@ char *
 get_message_for_content_type (const char *content_type)
 {
     char *message;
+    const GStrv known_types = (char *[])
+    {
+        "x-content/audio-cdda",
+        "x-content/audio-dvd",
+        "x-content/video-vcd",
+        "x-content/video-dvd",
+        "x-content/video-svcd",
+        "x-content/image-picturecd",
+        NULL,
+    };
 
     /* Customize greeting for well-known content types */
-    if (strcmp (content_type, "x-content/audio-cdda") == 0)
+    if (g_strv_contains ((const gchar **) known_types, content_type))
     {
-        /* translators: these describe the contents of removable media */
-        message = g_strdup (_("Audio CD"));
-    }
-    else if (strcmp (content_type, "x-content/audio-dvd") == 0)
-    {
-        message = g_strdup (_("Audio DVD"));
-    }
-    else if (strcmp (content_type, "x-content/video-dvd") == 0)
-    {
-        message = g_strdup (_("Video DVD"));
-    }
-    else if (strcmp (content_type, "x-content/video-vcd") == 0)
-    {
-        message = g_strdup (_("Video CD"));
-    }
-    else if (strcmp (content_type, "x-content/video-svcd") == 0)
-    {
-        message = g_strdup (_("Super Video CD"));
+        message = g_content_type_get_description (content_type);
     }
     else if (strcmp (content_type, "x-content/image-photocd") == 0)
     {
+        /* translators: these describe the contents of removable media */
         message = g_strdup (_("Photo CD"));
     }
-    else if (strcmp (content_type, "x-content/image-picturecd") == 0)
-    {
-        message = g_strdup (_("Picture CD"));
-    }
     else if (strcmp (content_type, "x-content/image-dcf") == 0)
     {
         message = g_strdup (_("Contains digital photos"));
@@ -999,6 +992,7 @@ nautilus_ensure_extension_builtins (void)
      * that they will be registered by the time the extension point
      * is iterating over its extensions.
      */
+    g_type_ensure (NAUTILUS_TYPE_NETWORK_DIRECTORY);
     g_type_ensure (NAUTILUS_TYPE_SEARCH_DIRECTORY);
     g_type_ensure (NAUTILUS_TYPE_STARRED_DIRECTORY);
 }
@@ -1113,3 +1107,29 @@ check_schema_available (const gchar *schema_id)
 
     return TRUE;
 }
+
+gboolean
+is_external_volume (GVolume *volume)
+{
+    gboolean is_external;
+    GDrive *drive;
+    char *id;
+
+    drive = g_volume_get_drive (volume);
+    id = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_CLASS);
+
+    is_external = g_volume_can_eject (volume);
+
+    /* NULL volume identifier only happens on removable devices */
+    is_external |= !id;
+
+    if (drive)
+    {
+        is_external |= g_drive_is_removable (drive);
+    }
+
+    g_clear_object (&drive);
+    g_free (id);
+
+    return is_external;
+}
