@@ -17,11 +17,15 @@ struct _NautilusFilenameValidator
 
     NautilusDirectory *containing_directory;
     gboolean target_is_folder;
+    gboolean allow_overwrite;
     char *original_name;
 
     const char *feedback_text;
     char *new_name;
     gboolean passed;
+    gboolean valid_name;
+    gboolean duplicated_name;
+    gboolean will_overwrite;
 
     gboolean duplicated_is_folder;
     gint duplicated_label_timeout_id;
@@ -41,7 +45,9 @@ enum
     PROP_NEW_NAME,
     PROP_CONTAINING_DIRECTORY,
     PROP_PASSED,
+    PROP_ALLOW_OVERWRITE,
     PROP_TARGET_IS_FOLDER,
+    PROP_WILL_OVERWRITE,
     NUM_PROPERTIES
 };
 
@@ -123,33 +129,33 @@ nautilus_filename_validator_name_is_valid (NautilusFilenameValidator  *self,
     else if (strstr (name, "/") != NULL)
     {
         is_valid = FALSE;
-        *error_message = is_folder ? _("Folder names cannot contain “/”.") :
-                                     _("File names cannot contain “/”.");
+        *error_message = is_folder ? _("Folder names cannot contain “/”") :
+                                     _("File names cannot contain “/”");
     }
     else if (strcmp (name, ".") == 0)
     {
         is_valid = FALSE;
-        *error_message = is_folder ? _("A folder cannot be called “.”.") :
-                                     _("A file cannot be called “.”.");
+        *error_message = is_folder ? _("A folder cannot be called “.”") :
+                                     _("A file cannot be called “.”");
     }
     else if (strcmp (name, "..") == 0)
     {
         is_valid = FALSE;
-        *error_message = is_folder ? _("A folder cannot be called “..”.") :
-                                     _("A file cannot be called “..”.");
+        *error_message = is_folder ? _("A folder cannot be called “..”") :
+                                     _("A file cannot be called “..”");
     }
     else if (nautilus_filename_validator_is_name_too_long (self, name))
     {
         is_valid = FALSE;
-        *error_message = is_folder ? _("Folder name is too long.") :
-                                     _("File name is too long.");
+        *error_message = is_folder ? _("Folder name is too long") :
+                                     _("File name is too long");
     }
 
     if (is_valid && g_str_has_prefix (name, "."))
     {
         /* We must warn about the side effect */
-        *error_message = is_folder ? _("Folders with “.” at the beginning of their name are hidden.") :
-                                     _("Files with “.” at the beginning of their name are hidden.");
+        *error_message = is_folder ? _("Folders with “.” at the beginning of their name are hidden") :
+                                     _("Files with “.” at the beginning of their name are hidden");
     }
 
     return is_valid;
@@ -158,8 +164,8 @@ nautilus_filename_validator_name_is_valid (NautilusFilenameValidator  *self,
 static gboolean
 duplicated_file_label_show (NautilusFilenameValidator *self)
 {
-    const char *text = self->duplicated_is_folder ? _("A folder with that name already exists.") :
-                                                    _("A file with that name already exists.");
+    const char *text = self->duplicated_is_folder ? _("A folder with that name already exists") :
+                                                    _("A file with that name already exists");
 
     if (self->feedback_text != text)
     {
@@ -174,19 +180,19 @@ duplicated_file_label_show (NautilusFilenameValidator *self)
 }
 
 static void
-filename_validator_process_new_name (NautilusFilenameValidator *self,
-                                     gboolean                  *duplicated_name,
-                                     gboolean                  *valid_name)
+filename_validator_process_new_name (NautilusFilenameValidator *self)
 {
     g_return_if_fail (NAUTILUS_IS_DIRECTORY (self->containing_directory));
 
+    g_object_freeze_notify (G_OBJECT (self));
+
     g_autofree gchar *name = nautilus_filename_validator_get_new_name (self);
     gchar *error_message = NULL;
     g_autoptr (NautilusFile) existing_file = NULL;
 
-    *valid_name = nautilus_filename_validator_name_is_valid (self,
-                                                             name,
-                                                             &error_message);
+    self->valid_name = nautilus_filename_validator_name_is_valid (self,
+                                                                  name,
+                                                                  &error_message);
 
     if (self->feedback_text != error_message)
     {
@@ -196,22 +202,37 @@ filename_validator_process_new_name (NautilusFilenameValidator *self,
     }
 
     existing_file = nautilus_directory_get_file_by_name (self->containing_directory, name);
-    *duplicated_name = existing_file != NULL &&
-                       !nautilus_filename_validator_ignore_existing_file (self,
-                                                                          existing_file);
+    self->duplicated_name = existing_file != NULL &&
+                            !nautilus_filename_validator_ignore_existing_file (self,
+                                                                               existing_file);
 
-    if (self->passed != (*valid_name && !*duplicated_name))
+    gboolean passed = (self->valid_name &&
+                       (self->allow_overwrite || !self->duplicated_name));
+
+    if (self->passed != passed)
     {
-        self->passed = (*valid_name && !*duplicated_name);
+        self->passed = passed;
         g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PASSED]);
     }
 
+    gboolean will_overwrite = (self->allow_overwrite &&
+                               self->passed &&
+                               self->duplicated_name);
+
+    if (self->will_overwrite != will_overwrite)
+    {
+        self->will_overwrite = will_overwrite;
+        g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_WILL_OVERWRITE]);
+    }
+
     g_clear_handle_id (&self->duplicated_label_timeout_id, g_source_remove);
 
-    if (*duplicated_name)
+    if (self->duplicated_name)
     {
         self->duplicated_is_folder = nautilus_file_is_directory (existing_file);
     }
+
+    g_object_thaw_notify (G_OBJECT (self));
 }
 
 static void
@@ -220,18 +241,14 @@ on_directory_info_ready_to_validate (NautilusDirectory *directory,
                                      gpointer           user_data)
 {
     NautilusFilenameValidator *self = NAUTILUS_FILENAME_VALIDATOR (user_data);
-    gboolean duplicated_name;
-    gboolean valid_name;
 
-    filename_validator_process_new_name (self,
-                                         &duplicated_name,
-                                         &valid_name);
+    filename_validator_process_new_name (self);
 
     /* Report duplicated file only if not other message shown (for instance,
      * folders like "." or ".." will always exists, but we consider it as an
      * error, not as a duplicated file or if the name is the same as the file
      * we are renaming also don't report as a duplicated */
-    if (duplicated_name && valid_name)
+    if (self->duplicated_name && self->valid_name)
     {
         self->duplicated_label_timeout_id = g_timeout_add (FILE_NAME_DUPLICATED_LABEL_TIMEOUT,
                                                            (GSourceFunc) duplicated_file_label_show,
@@ -251,6 +268,12 @@ void
 nautilus_filename_validator_validate (NautilusFilenameValidator *self)
 {
     g_return_if_fail (NAUTILUS_IS_FILENAME_VALIDATOR (self));
+
+    if (self->containing_directory == NULL)
+    {
+        return;
+    }
+
     g_return_if_fail (NAUTILUS_IS_DIRECTORY (self->containing_directory));
 
     nautilus_directory_call_when_ready (self->containing_directory,
@@ -266,14 +289,10 @@ on_directory_info_ready_to_try_accept (NautilusDirectory *directory,
                                        gpointer           user_data)
 {
     NautilusFilenameValidator *self = NAUTILUS_FILENAME_VALIDATOR (user_data);
-    gboolean duplicated_name;
-    gboolean valid_name;
 
-    filename_validator_process_new_name (self,
-                                         &duplicated_name,
-                                         &valid_name);
+    filename_validator_process_new_name (self);
 
-    if (valid_name && !duplicated_name)
+    if (self->passed)
     {
         g_signal_emit (self, signals[NAME_ACCEPTED], 0);
     }
@@ -282,7 +301,7 @@ on_directory_info_ready_to_try_accept (NautilusDirectory *directory,
         /* Report duplicated file only if not other message shown (for instance,
          * folders like "." or ".." will always exists, but we consider it as an
          * error, not as a duplicated file) */
-        if (duplicated_name && valid_name)
+        if (self->duplicated_name && self->valid_name)
         {
             /* Show it inmediatily since the user tried to trigger the action */
             duplicated_file_label_show (self);
@@ -312,6 +331,12 @@ nautilus_filename_validator_try_accept (NautilusFilenameValidator *self)
                                         self);
 }
 
+gboolean
+nautilus_filename_validator_get_has_feedback (NautilusFilenameValidator *self)
+{
+    return (self->feedback_text != NULL);
+}
+
 static void
 nautilus_filename_validator_init (NautilusFilenameValidator *self)
 {
@@ -335,7 +360,7 @@ nautilus_filename_validator_get_property (GObject    *object,
 
         case PROP_HAS_FEEDBACK:
         {
-            g_value_set_boolean (value, (self->feedback_text != NULL));
+            g_value_set_boolean (value, nautilus_filename_validator_get_has_feedback (self));
         }
         break;
 
@@ -352,6 +377,12 @@ nautilus_filename_validator_get_property (GObject    *object,
         }
         break;
 
+        case PROP_WILL_OVERWRITE:
+        {
+            g_value_set_boolean (value, self->will_overwrite);
+        }
+        break;
+
         default:
         {
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -384,6 +415,12 @@ nautilus_filename_validator_set_property (GObject      *object,
         }
         break;
 
+        case PROP_ALLOW_OVERWRITE:
+        {
+            self->allow_overwrite = g_value_get_boolean (value);
+        }
+        break;
+
         case PROP_TARGET_IS_FOLDER:
         {
             self->target_is_folder = g_value_get_boolean (value);
@@ -459,10 +496,24 @@ nautilus_filename_validator_class_init (NautilusFilenameValidatorClass *klass)
         g_param_spec_object ("containing-directory", NULL, NULL,
                              NAUTILUS_TYPE_DIRECTORY,
                              G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_ALLOW_OVERWRITE] =
+        g_param_spec_boolean ("allow-overwrite", NULL, NULL,
+                              FALSE,
+                              G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
     properties[PROP_TARGET_IS_FOLDER] =
         g_param_spec_boolean ("target-is-folder", NULL, NULL,
                               FALSE,
                               G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_WILL_OVERWRITE] =
+        g_param_spec_boolean ("will-overwrite", NULL, NULL,
+                              FALSE,
+                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
     g_object_class_install_properties (object_class, NUM_PROPERTIES, properties);
 }
+
+gboolean
+nautilus_filename_validator_get_will_overwrite (NautilusFilenameValidator *self)
+{
+    return self->will_overwrite;
+}
