@@ -14,7 +14,9 @@
 #define NAUTILUS_TYPE_FILENAME_VALIDATOR nautilus_filename_validator_get_type ()
 G_DECLARE_FINAL_TYPE (NautilusFilenameValidator, nautilus_filename_validator, NAUTILUS, FILENAME_VALIDATOR, GObject)
 
+gboolean nautilus_filename_validator_get_has_feedback        (NautilusFilenameValidator *self);
 gchar * nautilus_filename_validator_get_new_name             (NautilusFilenameValidator *self);
+gboolean nautilus_filename_validator_get_will_overwrite      (NautilusFilenameValidator *self);
 
 void    nautilus_filename_validator_set_containing_directory (NautilusFilenameValidator *self,
                                                               NautilusDirectory         *directory);
