@@ -54,6 +54,8 @@ G_BEGIN_DECLS
 #define NAUTILUS_WINDOW_STATE_INITIAL_SIZE			"initial-size"
 #define NAUTILUS_WINDOW_STATE_MAXIMIZED				"maximized"
 
+#define NAUTILUS_WINDOW_STATE_INITIAL_SIZE_FILE_CHOOSER         "initial-size-file-chooser"
+
 /* Sorting order */
 #define NAUTILUS_PREFERENCES_SORT_DIRECTORIES_FIRST		"sort-directories-first"
 #define NAUTILUS_PREFERENCES_DEFAULT_SORT_ORDER			"default-sort-order"
@@ -137,6 +139,9 @@ typedef enum
 } NautilusDateTimeFormat;
 
 void nautilus_global_preferences_init                      (void);
+guint nautilus_global_preferences_get_back_button (void);
+guint nautilus_global_preferences_get_forward_button (void);
+gboolean nautilus_global_preferences_get_use_extra_buttons (void);
 
 extern GSettings *nautilus_preferences;
 extern GSettings *nautilus_compression_preferences;
@@ -147,5 +152,6 @@ extern GSettings *gtk_filechooser_preferences;
 extern GSettings *gnome_lockdown_preferences;
 extern GSettings *gnome_interface_preferences;
 extern GSettings *gnome_privacy_preferences;
+extern GSettings *localsearch_preferences;
 
 G_END_DECLS
