@@ -30,6 +30,7 @@
 #include "nautilus-location-banner.h"
 #include "nautilus-enum-types.h"
 #include "nautilus-scheme.h"
+#include "nautilus-trash-monitor.h"
 
 #define FILE_SHARING_SCHEMA_ID "org.gnome.desktop.file-sharing"
 
@@ -177,16 +178,24 @@ get_mode_for_location (GFile *location)
     }
 }
 
+static void
+update_trash_banner_visibility (AdwBanner *banner)
+{
+    adw_banner_set_revealed (banner, !nautilus_trash_monitor_is_empty ());
+}
+
 static void
 set_mode (AdwBanner                  *banner,
           NautilusLocationBannerMode  mode)
 {
     const char *button_label = NULL;
     GCallback callback = NULL;
+    gboolean banner_is_revealed = TRUE;
 
     /* Reset signal handlers. */
     g_signal_handlers_disconnect_by_data (banner, &nautilus_location_banner_load);
     g_signal_handlers_disconnect_by_data (gnome_privacy_preferences, banner);
+    g_signal_handlers_disconnect_by_data (nautilus_trash_monitor_get (), banner);
 
     switch (mode)
     {
@@ -222,9 +231,18 @@ set_mode (AdwBanner                  *banner,
         case NAUTILUS_LOCATION_BANNER_TRASH:
         {
             adw_banner_set_title (banner, "");
+            if (nautilus_trash_monitor_is_empty ())
+            {
+                banner_is_revealed = FALSE;
+            }
+
             button_label = _("_Empty Trash…");
             callback = G_CALLBACK (on_trash_clear_clicked);
 
+            g_signal_connect_swapped (nautilus_trash_monitor_get (),
+                                      "trash-state-changed",
+                                      G_CALLBACK (update_trash_banner_visibility),
+                                      banner);
             g_signal_connect_object (gnome_privacy_preferences,
                                      "changed::remove-old-trash-files",
                                      G_CALLBACK (on_remove_old_trash_files_changed),
@@ -256,7 +274,7 @@ set_mode (AdwBanner                  *banner,
     }
 
     adw_banner_set_button_label (banner, button_label);
-    adw_banner_set_revealed (banner, TRUE);
+    adw_banner_set_revealed (banner, banner_is_revealed);
 
     if (callback != NULL)
     {
