@@ -57,6 +57,8 @@ typedef enum
     MOUNT_BUTTON,
     TRASH_BUTTON,
     NETWORK_BUTTON,
+    COMPUTER_BUTTON,
+    BURN_BUTTON,
 } ButtonType;
 
 #define BUTTON_DATA(x) ((ButtonData *) (x))
@@ -649,7 +651,8 @@ static void
 pop_up_pathbar_context_menu (NautilusPathBar *self,
                              NautilusFile    *file)
 {
-    if (file != NULL)
+    if (file != NULL &&
+        nautilus_window_slot_get_mode (self->slot) == NAUTILUS_MODE_BROWSE)
     {
         schedule_pop_up_context_menu (self, file);
     }
@@ -1074,11 +1077,22 @@ setup_button_type (ButtonData      *button_data,
         button_data->type = TRASH_BUTTON;
         button_data->is_root = TRUE;
     }
-    else if (nautilus_is_root_for_scheme (location, SCHEME_NETWORK_VIEW))
+    else if (nautilus_is_root_for_scheme (location, SCHEME_NETWORK_VIEW) ||
+             nautilus_is_root_for_scheme (location, SCHEME_NETWORK))
     {
         button_data->type = NETWORK_BUTTON;
         button_data->is_root = TRUE;
     }
+    else if (nautilus_is_root_for_scheme (location, SCHEME_COMPUTER))
+    {
+        button_data->type = COMPUTER_BUTTON;
+        button_data->is_root = TRUE;
+    }
+    else if (nautilus_is_root_for_scheme (location, SCHEME_BURN))
+    {
+        button_data->type = BURN_BUTTON;
+        button_data->is_root = TRUE;
+    }
     else
     {
         button_data->type = NORMAL_BUTTON;
@@ -1217,12 +1231,19 @@ make_button_data (NautilusPathBar *self,
         case RECENT_BUTTON:
         case STARRED_BUTTON:
         case NETWORK_BUTTON:
+        case COMPUTER_BUTTON:
+        case BURN_BUTTON:
         {
             button_data->label = gtk_label_new (NULL);
             child = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
             button_data->container = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
             gtk_box_append (GTK_BOX (button_data->container), button_data->button);
 
+            gtk_accessible_update_relation (GTK_ACCESSIBLE (button_data->image),
+                                            GTK_ACCESSIBLE_RELATION_LABELLED_BY,
+                                            button_data->label, NULL,
+                                            -1);
+
             gtk_box_append (GTK_BOX (child), button_data->image);
             gtk_box_append (GTK_BOX (child), button_data->label);
         }
