@@ -20,8 +20,7 @@
  * Author: Cosimo Cecchi <cosimoc@redhat.com>
  */
 
-#ifndef __NAUTILUS_PROGRESS_INFO_MANAGER_H__
-#define __NAUTILUS_PROGRESS_INFO_MANAGER_H__
+#pragma once
 
 #include <glib-object.h>
 
@@ -43,5 +42,3 @@ void nautilus_progress_manager_remove_viewer (NautilusProgressInfoManager *self,
 gboolean nautilus_progress_manager_has_viewers (NautilusProgressInfoManager *self);
 
 G_END_DECLS
-
-#endif /* __NAUTILUS_PROGRESS_INFO_MANAGER_H__ */
