@@ -608,6 +608,7 @@ search_engine_hits_added (NautilusSearchEngine    *engine,
     GList *hit_list;
     GList *file_list;
     NautilusFile *file;
+    g_autoptr (GDateTime) now = g_date_time_new_now_local ();
     SearchMonitor *monitor;
     GList *monitor_list;
 
@@ -620,7 +621,7 @@ search_engine_hits_added (NautilusSearchEngine    *engine,
 
         uri = nautilus_search_hit_get_uri (hit);
 
-        nautilus_search_hit_compute_scores (hit, self->query);
+        nautilus_search_hit_compute_scores (hit, now, self->query);
 
         file = nautilus_file_get_by_uri (uri);
         nautilus_file_set_search_relevance (file, nautilus_search_hit_get_relevance (hit));
@@ -691,6 +692,24 @@ search_engine_finished (NautilusSearchEngine         *engine,
     }
 }
 
+static NautilusFile *
+search_new_file_from_filename (NautilusDirectory *directory,
+                               const char        *filename,
+                               gboolean           self_owned)
+{
+    if (!self_owned)
+    {
+        /* This doesn't normally happen, unless the user somehow types in a uri
+         * that references a file like this.
+         * See https://bugzilla.gnome.org/show_bug.cgi?id=349840 */
+        return NAUTILUS_DIRECTORY_CLASS (nautilus_search_directory_parent_class)->new_file_from_filename (directory, filename, self_owned);
+    }
+
+    return NAUTILUS_FILE (g_object_new (NAUTILUS_TYPE_SEARCH_DIRECTORY_FILE,
+                                        "directory", directory,
+                                        NULL));
+}
+
 static void
 search_force_reload (NautilusDirectory *directory)
 {
@@ -929,6 +948,8 @@ nautilus_search_directory_class_init (NautilusSearchDirectoryClass *class)
     oclass->get_property = search_get_property;
     oclass->set_property = search_set_property;
 
+    directory_class->new_file_from_filename = search_new_file_from_filename;
+
     directory_class->are_all_files_seen = search_are_all_files_seen;
     directory_class->contains_file = search_contains_file;
     directory_class->force_reload = search_force_reload;
