@@ -18,6 +18,7 @@
  * Author: Alexander Larsson <alexl@redhat.com>
  *
  */
+#define G_LOG_DOMAIN "nautilus-search"
 
 #include <config.h>
 #include "nautilus-search-hit.h"
@@ -27,8 +28,6 @@
 #include "nautilus-directory-private.h"
 #include "nautilus-file.h"
 #include "nautilus-ui-utilities.h"
-#define DEBUG_FLAG NAUTILUS_DEBUG_SEARCH
-#include "nautilus-debug.h"
 
 #include <string.h>
 #include <glib.h>
@@ -94,7 +93,7 @@ search_finished (NautilusSearchEngineModel *model)
 
     if (model->hits != NULL)
     {
-        DEBUG ("Model engine hits added");
+        g_debug ("Model engine hits added");
         nautilus_search_provider_hits_added (NAUTILUS_SEARCH_PROVIDER (model),
                                              model->hits);
         g_list_free_full (model->hits, g_object_unref);
@@ -105,7 +104,7 @@ search_finished (NautilusSearchEngineModel *model)
 
     g_object_notify (G_OBJECT (model), "running");
 
-    DEBUG ("Model engine finished");
+    g_debug ("Model engine finished");
     nautilus_search_provider_finished (NAUTILUS_SEARCH_PROVIDER (model),
                                        NAUTILUS_SEARCH_PROVIDER_STATUS_NORMAL);
     g_object_unref (model);
@@ -154,8 +153,8 @@ model_directory_ready_cb (NautilusDirectory *directory,
 
         file = l->data;
 
-        display_name = nautilus_file_get_display_name (file);
-        match = nautilus_query_matches_string (model->query, display_name);
+        match = nautilus_query_matches_string (model->query,
+                                               nautilus_file_get_display_name (file));
         found = (match > -1);
         if (!found)
         {
@@ -166,7 +165,7 @@ model_directory_ready_cb (NautilusDirectory *directory,
         {
             found = FALSE;
 
-            for (gint i = 0; i < mime_types->len; i++)
+            for (guint i = 0; i < mime_types->len; i++)
             {
                 if (nautilus_file_is_mime_type (file, g_ptr_array_index (mime_types, i)))
                 {
@@ -259,7 +258,7 @@ nautilus_search_engine_model_start (NautilusSearchProvider *provider)
         return;
     }
 
-    DEBUG ("Model engine start");
+    g_debug ("Model engine start");
 
     g_object_ref (model);
     model->query_pending = TRUE;
@@ -286,7 +285,7 @@ nautilus_search_engine_model_stop (NautilusSearchProvider *provider)
 
     if (model->query_pending)
     {
-        DEBUG ("Model engine stop");
+        g_debug ("Model engine stop");
 
         nautilus_directory_cancel_callback (model->directory,
                                             model_directory_ready_cb, model);
