@@ -375,7 +375,10 @@ nautilus_tag_manager_star_files (NautilusTagManager  *self,
     GTask *task;
     UpdateData *update_data;
 
-    g_debug ("Starring %i files", g_list_length (selection));
+    if (g_getenv ("G_MESSAGES_DEBUG") != NULL)
+    {
+        g_debug ("Starring %i files", g_list_length (selection));
+    }
 
     task = g_task_new (object, cancellable, callback, NULL);
 
@@ -408,7 +411,10 @@ nautilus_tag_manager_unstar_files (NautilusTagManager  *self,
     GTask *task;
     UpdateData *update_data;
 
-    g_debug ("Unstarring %i files", g_list_length (selection));
+    if (g_getenv ("G_MESSAGES_DEBUG") != NULL)
+    {
+        g_debug ("Unstarring %i files", g_list_length (selection));
+    }
 
     task = g_task_new (object, cancellable, callback, NULL);
 
@@ -483,7 +489,7 @@ on_tracker_notifier_events (TrackerNotifier *notifier,
             if (inserted)
             {
                 g_debug ("Added %s to starred files list", file_url);
-                changed_file = nautilus_file_get_existing_by_uri (file_url);
+                changed_file = nautilus_file_get_by_uri (file_url);
             }
         }
         else
@@ -493,7 +499,7 @@ on_tracker_notifier_events (TrackerNotifier *notifier,
             if (removed)
             {
                 g_debug ("Removed %s from starred files list", file_url);
-                changed_file = nautilus_file_get_existing_by_uri (file_url);
+                changed_file = nautilus_file_get_by_uri (file_url);
             }
         }
 
