@@ -33,18 +33,14 @@
 #include "nautilus-pathbar.h"
 #include "nautilus-view-controls.h"
 #include "nautilus-ui-utilities.h"
-#include "nautilus-window.h"
 
 struct _NautilusToolbar
 {
     AdwBin parent_instance;
 
-    NautilusWindow *window;
-
     GtkWidget *history_controls_stack;
     GtkWidget *history_controls;
     GtkWidget *history_controls_placeholder;
-    GtkWidget *path_bar_container;
     GtkWidget *location_entry_container;
     GtkWidget *search_container;
     GtkWidget *toolbar_switcher;
@@ -55,14 +51,15 @@ struct _NautilusToolbar
     GtkWidget *search_button_placeholder;
 
     gboolean show_location_entry;
+    GtkWidget *focus_before_location_entry;
 
     GtkWidget *sidebar_button;
+    gboolean show_new_folder_button;
     gboolean show_sidebar_button;
     gboolean sidebar_button_active;
 
-    gboolean show_toolbar_children;
-
-    GtkWidget *location_entry_close_button;
+    gboolean show_view_controls;
+    gboolean show_history_controls;
 
     /* active slot & bindings */
     NautilusWindowSlot *window_slot;
@@ -71,11 +68,12 @@ struct _NautilusToolbar
 enum
 {
     PROP_0,
-    PROP_SHOW_LOCATION_ENTRY,
     PROP_WINDOW_SLOT,
+    PROP_SHOW_NEW_FOLDER_BUTTON,
     PROP_SHOW_SIDEBAR_BUTTON,
     PROP_SIDEBAR_BUTTON_ACTIVE,
-    PROP_SHOW_TOOLBAR_CHILDREN,
+    PROP_SHOW_VIEW_CONTROLS,
+    PROP_SHOW_HISTORY_CONTROLS,
     NUM_PROPERTIES
 };
 
@@ -118,10 +116,69 @@ toolbar_update_appearance (NautilusToolbar *self)
 }
 
 static void
-on_location_entry_close (GtkWidget       *close_button,
-                         NautilusToolbar *self)
+nautilus_toolbar_open_location_entry (NautilusToolbar *self,
+                                      const char      *special_text)
 {
-    nautilus_toolbar_set_show_location_entry (self, FALSE);
+    if (self->show_location_entry)
+    {
+        return;
+    }
+
+    /* Remember focus widget. */
+    GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
+    GtkWidget *focus_widget = (root != NULL) ? gtk_root_get_focus (root) : NULL;
+
+    g_set_weak_pointer (&self->focus_before_location_entry, focus_widget);
+
+    self->show_location_entry = TRUE;
+    toolbar_update_appearance (self);
+
+    gtk_widget_grab_focus (self->location_entry);
+
+    if (special_text != NULL)
+    {
+        nautilus_location_entry_set_special_text (NAUTILUS_LOCATION_ENTRY (self->location_entry),
+                                                  special_text);
+        gtk_editable_set_position (GTK_EDITABLE (self->location_entry), -1);
+    }
+}
+
+static void
+nautilus_toolbar_close_location_entry (NautilusToolbar *self)
+{
+    if (!self->show_location_entry)
+    {
+        return;
+    }
+
+    self->show_location_entry = FALSE;
+    toolbar_update_appearance (self);
+
+    if (self->focus_before_location_entry != NULL)
+    {
+        /* Restore focus widget. */
+        gtk_widget_grab_focus (self->focus_before_location_entry);
+        g_clear_weak_pointer (&self->focus_before_location_entry);
+    }
+}
+
+static void
+on_path_bar_open_location (NautilusPathBar   *path_bar,
+                           GFile             *location,
+                           NautilusOpenFlags  open_flags,
+                           gpointer           user_data)
+{
+    NautilusToolbar *self = NAUTILUS_TOOLBAR (user_data);
+
+    if (open_flags & (NAUTILUS_OPEN_FLAG_NEW_WINDOW | NAUTILUS_OPEN_FLAG_NEW_TAB))
+    {
+        nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
+                                                 location, open_flags, NULL, NULL, NULL);
+    }
+    else
+    {
+        nautilus_window_slot_open_location_full (self->window_slot, location, open_flags, NULL);
+    }
 }
 
 static void
@@ -149,39 +206,48 @@ on_location_entry_focus_leave (GtkEventControllerFocus *controller,
         return;
     }
 
-    nautilus_toolbar_set_show_location_entry (toolbar, FALSE);
+    nautilus_toolbar_close_location_entry (toolbar);
 }
 
 static void
-nautilus_toolbar_constructed (GObject *object)
+on_location_entry_location_changed (NautilusLocationEntry *entry,
+                                    GFile                 *location,
+                                    gpointer               user_data)
 {
-    NautilusToolbar *self = NAUTILUS_TOOLBAR (object);
-    GtkEventController *controller;
+    NautilusToolbar *self = NAUTILUS_TOOLBAR (user_data);
 
-    self->path_bar = GTK_WIDGET (g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL));
-    gtk_box_append (GTK_BOX (self->path_bar_container),
-                    self->path_bar);
+    nautilus_toolbar_close_location_entry (self);
+    nautilus_window_slot_open_location_full (self->window_slot, location, 0, NULL);
+}
 
-    self->location_entry = nautilus_location_entry_new ();
-    gtk_box_append (GTK_BOX (self->location_entry_container),
-                    self->location_entry);
-    self->location_entry_close_button = gtk_button_new_from_icon_name ("window-close-symbolic");
-    gtk_widget_set_tooltip_text (self->location_entry_close_button, _("Cancel"));
-    gtk_box_append (GTK_BOX (self->location_entry_container),
-                    self->location_entry_close_button);
-    g_signal_connect (self->location_entry_close_button, "clicked",
-                      G_CALLBACK (on_location_entry_close), self);
+static void
+action_edit_location (GtkWidget  *widget,
+                      const char *action_name,
+                      GVariant   *parameter)
+{
+    NautilusToolbar *self = NAUTILUS_TOOLBAR (widget);
 
-    controller = gtk_event_controller_focus_new ();
-    gtk_widget_add_controller (self->location_entry, controller);
-    g_signal_connect (controller, "leave",
-                      G_CALLBACK (on_location_entry_focus_leave), self);
+    nautilus_toolbar_open_location_entry (self, NULL);
+}
 
-    /* Setting a max width on one entry to effectively set a max expansion for
-     * the whole title widget. */
-    gtk_editable_set_max_width_chars (GTK_EDITABLE (self->location_entry), 88);
+static void
+action_prompt_root_location (GtkWidget  *widget,
+                             const char *action_name,
+                             GVariant   *parameter)
+{
+    NautilusToolbar *self = NAUTILUS_TOOLBAR (widget);
 
-    toolbar_update_appearance (self);
+    nautilus_toolbar_open_location_entry (self, "/");
+}
+
+static void
+action_prompt_home_location (GtkWidget  *widget,
+                             const char *action_name,
+                             GVariant   *parameter)
+{
+    NautilusToolbar *self = NAUTILUS_TOOLBAR (widget);
+
+    nautilus_toolbar_open_location_entry (self, "~");
 }
 
 static void
@@ -189,8 +255,51 @@ nautilus_toolbar_init (NautilusToolbar *self)
 {
     g_type_ensure (NAUTILUS_TYPE_HISTORY_CONTROLS);
     g_type_ensure (NAUTILUS_TYPE_VIEW_CONTROLS);
+    g_type_ensure (NAUTILUS_TYPE_PATH_BAR);
+    g_type_ensure (NAUTILUS_TYPE_LOCATION_ENTRY);
 
     gtk_widget_init_template (GTK_WIDGET (self));
+
+    GtkShortcutController *shortcuts = GTK_SHORTCUT_CONTROLLER (gtk_shortcut_controller_new ());
+
+    gtk_shortcut_controller_set_scope (shortcuts, GTK_SHORTCUT_SCOPE_MANAGED);
+    gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (shortcuts));
+
+#define ADD_SHORTCUT_FOR_ACTION(controller, action, trigger) \
+        (gtk_shortcut_controller_add_shortcut ((controller), \
+                                               gtk_shortcut_new (gtk_shortcut_trigger_parse_string ((trigger)), \
+                                                                 gtk_named_action_new ((action)))))
+
+    ADD_SHORTCUT_FOR_ACTION (shortcuts, "toolbar.edit-location", "<control>l|Go|OpenURL");
+    ADD_SHORTCUT_FOR_ACTION (shortcuts, "toolbar.prompt-root-location", "slash|KP_Divide");
+    /* Support keyboard layouts which have a dead tilde key but not a tilde key. */
+    ADD_SHORTCUT_FOR_ACTION (shortcuts, "toolbar.prompt-home-location", "asciitilde|dead_tilde");
+
+#undef ADD_SHORTCUT_FOR_ACTION
+
+    /* Setup path bar */
+    g_signal_connect_object (self->path_bar, "open-location",
+                             G_CALLBACK (on_path_bar_open_location), self,
+                             G_CONNECT_DEFAULT);
+
+    /* Setup location entry */
+    GtkEventController *controller = gtk_event_controller_focus_new ();
+
+    gtk_widget_add_controller (self->location_entry, controller);
+    g_signal_connect (controller, "leave",
+                      G_CALLBACK (on_location_entry_focus_leave), self);
+
+    /* Setting a max width on one entry to effectively set a max expansion for
+     * the whole title widget. */
+    gtk_editable_set_max_width_chars (GTK_EDITABLE (self->location_entry), 88);
+    g_signal_connect_object (self->location_entry, "location-changed",
+                             G_CALLBACK (on_location_entry_location_changed), self,
+                             G_CONNECT_DEFAULT);
+    g_signal_connect_object (self->location_entry, "cancel",
+                             G_CALLBACK (nautilus_toolbar_close_location_entry), self,
+                             G_CONNECT_SWAPPED);
+
+    toolbar_update_appearance (self);
 }
 
 static void
@@ -203,18 +312,18 @@ nautilus_toolbar_get_property (GObject    *object,
 
     switch (property_id)
     {
-        case PROP_SHOW_LOCATION_ENTRY:
-        {
-            g_value_set_boolean (value, self->show_location_entry);
-        }
-        break;
-
         case PROP_WINDOW_SLOT:
         {
             g_value_set_object (value, self->window_slot);
         }
         break;
 
+        case PROP_SHOW_NEW_FOLDER_BUTTON:
+        {
+            g_value_set_boolean (value, self->show_new_folder_button);
+        }
+        break;
+
         case PROP_SHOW_SIDEBAR_BUTTON:
         {
             g_value_set_boolean (value, self->show_sidebar_button);
@@ -227,9 +336,15 @@ nautilus_toolbar_get_property (GObject    *object,
         }
         break;
 
-        case PROP_SHOW_TOOLBAR_CHILDREN:
+        case PROP_SHOW_VIEW_CONTROLS:
         {
-            g_value_set_boolean (value, self->show_toolbar_children);
+            g_value_set_boolean (value, self->show_view_controls);
+        }
+        break;
+
+        case PROP_SHOW_HISTORY_CONTROLS:
+        {
+            g_value_set_boolean (value, self->show_history_controls);
         }
         break;
 
@@ -262,18 +377,18 @@ nautilus_toolbar_set_property (GObject      *object,
 
     switch (property_id)
     {
-        case PROP_SHOW_LOCATION_ENTRY:
-        {
-            nautilus_toolbar_set_show_location_entry (self, g_value_get_boolean (value));
-        }
-        break;
-
         case PROP_WINDOW_SLOT:
         {
             nautilus_toolbar_set_window_slot (self, g_value_get_object (value));
         }
         break;
 
+        case PROP_SHOW_NEW_FOLDER_BUTTON:
+        {
+            self->show_new_folder_button = g_value_get_boolean (value);
+        }
+        break;
+
         case PROP_SHOW_SIDEBAR_BUTTON:
         {
             self->show_sidebar_button = g_value_get_boolean (value);
@@ -286,9 +401,15 @@ nautilus_toolbar_set_property (GObject      *object,
         }
         break;
 
-        case PROP_SHOW_TOOLBAR_CHILDREN:
+        case PROP_SHOW_VIEW_CONTROLS:
         {
-            self->show_toolbar_children = g_value_get_boolean (value);
+            self->show_view_controls = g_value_get_boolean (value);
+        }
+        break;
+
+        case PROP_SHOW_HISTORY_CONTROLS:
+        {
+            self->show_history_controls = g_value_get_boolean (value);
         }
         break;
 
@@ -316,6 +437,8 @@ nautilus_toolbar_finalize (GObject *obj)
 {
     NautilusToolbar *self = NAUTILUS_TOOLBAR (obj);
 
+    g_clear_weak_pointer (&self->focus_before_location_entry);
+
     g_signal_handlers_disconnect_by_func (nautilus_preferences,
                                           toolbar_update_appearance, self);
 
@@ -342,14 +465,6 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
     oclass->set_property = nautilus_toolbar_set_property;
     oclass->dispose = nautilus_toolbar_dispose;
     oclass->finalize = nautilus_toolbar_finalize;
-    oclass->constructed = nautilus_toolbar_constructed;
-
-    properties[PROP_SHOW_LOCATION_ENTRY] =
-        g_param_spec_boolean ("show-location-entry",
-                              "Whether to show the location entry",
-                              "Whether to show the location entry instead of the pathbar",
-                              FALSE,
-                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
     properties [PROP_WINDOW_SLOT] =
         g_param_spec_object ("window-slot",
@@ -359,14 +474,21 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
                              (G_PARAM_READWRITE |
                               G_PARAM_STATIC_STRINGS));
 
+    properties[PROP_SHOW_NEW_FOLDER_BUTTON] =
+        g_param_spec_boolean ("show-new-folder-button", NULL, NULL,
+                              FALSE,
+                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
     properties[PROP_SHOW_SIDEBAR_BUTTON] =
         g_param_spec_boolean ("show-sidebar-button", NULL, NULL, FALSE,
                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
     properties[PROP_SIDEBAR_BUTTON_ACTIVE] =
         g_param_spec_boolean ("sidebar-button-active", NULL, NULL, FALSE,
                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-    properties[PROP_SHOW_TOOLBAR_CHILDREN] =
-        g_param_spec_boolean ("show-toolbar-children", NULL, NULL, TRUE,
+    properties[PROP_SHOW_HISTORY_CONTROLS] =
+        g_param_spec_boolean ("show-history-controls", NULL, NULL, TRUE,
+                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_SHOW_VIEW_CONTROLS] =
+        g_param_spec_boolean ("show-view-controls", NULL, NULL, TRUE,
                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
     g_object_class_install_properties (oclass, NUM_PROPERTIES, properties);
@@ -379,13 +501,19 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, history_controls_placeholder);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, toolbar_switcher);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_container);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, path_bar_container);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, location_entry_container);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, path_bar);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, location_entry);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_button_stack);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_button_placeholder);
 
+    gtk_widget_class_bind_template_callback (widget_class, nautilus_toolbar_close_location_entry);
+
     gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_TOOLBAR);
+
+    gtk_widget_class_install_action (widget_class, "toolbar.edit-location", NULL, action_edit_location);
+    gtk_widget_class_install_action (widget_class, "toolbar.prompt-root-location", NULL, action_prompt_root_location);
+    gtk_widget_class_install_action (widget_class, "toolbar.prompt-home-location", NULL, action_prompt_home_location);
 }
 
 GtkWidget *
@@ -395,31 +523,6 @@ nautilus_toolbar_new (void)
                          NULL);
 }
 
-GtkWidget *
-nautilus_toolbar_get_path_bar (NautilusToolbar *self)
-{
-    return self->path_bar;
-}
-
-GtkWidget *
-nautilus_toolbar_get_location_entry (NautilusToolbar *self)
-{
-    return self->location_entry;
-}
-
-void
-nautilus_toolbar_set_show_location_entry (NautilusToolbar *self,
-                                          gboolean         show_location_entry)
-{
-    if (show_location_entry != self->show_location_entry)
-    {
-        self->show_location_entry = show_location_entry;
-        toolbar_update_appearance (self);
-
-        g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SHOW_LOCATION_ENTRY]);
-    }
-}
-
 static void
 box_remove_all_children (GtkBox *box)
 {
@@ -430,6 +533,22 @@ box_remove_all_children (GtkBox *box)
     }
 }
 
+static void
+on_slot_location_changed (NautilusToolbar *self)
+{
+    g_assert (self->window_slot != NULL);
+
+    GFile *location = nautilus_window_slot_get_location (self->window_slot);
+
+    if (location == NULL)
+    {
+        return;
+    }
+
+    nautilus_location_entry_set_location (NAUTILUS_LOCATION_ENTRY (self->location_entry), location);
+    nautilus_path_bar_set_path (NAUTILUS_PATH_BAR (self->path_bar), location);
+}
+
 static void
 slot_on_extensions_background_menu_changed (NautilusToolbar    *self,
                                             GParamSpec         *param,
@@ -454,6 +573,12 @@ slot_on_templates_menu_changed (NautilusToolbar    *self,
                                           menu);
 }
 
+void
+nautilus_toolbar_show_current_location_menu (NautilusToolbar *self)
+{
+    nautilus_path_bar_show_current_location_menu (NAUTILUS_PATH_BAR (self->path_bar));
+}
+
 /* Called from on_window_slot_destroyed(), since bindings and signal handlers
  * are automatically removed once the slot goes away.
  */
@@ -461,8 +586,6 @@ static void
 nautilus_toolbar_set_window_slot_real (NautilusToolbar    *self,
                                        NautilusWindowSlot *slot)
 {
-    g_autoptr (GList) children = NULL;
-
     self->window_slot = slot;
 
     if (self->window_slot != NULL)
@@ -471,6 +594,10 @@ nautilus_toolbar_set_window_slot_real (NautilusToolbar    *self,
                            on_window_slot_destroyed,
                            self);
 
+        on_slot_location_changed (self);
+
+        g_signal_connect_swapped (self->window_slot, "notify::location",
+                                  G_CALLBACK (on_slot_location_changed), self);
         g_signal_connect_swapped (self->window_slot, "notify::extensions-background-menu",
                                   G_CALLBACK (slot_on_extensions_background_menu_changed), self);
         g_signal_connect_swapped (self->window_slot, "notify::templates-menu",
