@@ -441,7 +441,7 @@ compare_data_func (gconstpointer a,
                    gconstpointer b,
                    gpointer      user_data)
 {
-    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (user_data);
+    NautilusViewModel *self = (NautilusViewModel *) user_data;
 
     if (nautilus_view_model_get_sorter (self) == NULL)
     {
@@ -492,7 +492,7 @@ nautilus_view_model_get_sorter (NautilusViewModel *self)
 {
     GtkTreeListRowSorter *row_sorter;
 
-    row_sorter = GTK_TREE_LIST_ROW_SORTER (gtk_sort_list_model_get_sorter (self->sort_model));
+    row_sorter = (GtkTreeListRowSorter *) gtk_sort_list_model_get_sorter (self->sort_model);
 
     return row_sorter != NULL ? gtk_tree_list_row_sorter_get_sorter (row_sorter) : NULL;
 }
@@ -564,58 +564,52 @@ nautilus_view_model_get_item_for_file (NautilusViewModel *self,
 
 void
 nautilus_view_model_remove_items (NautilusViewModel *self,
-                                  GList             *items,
+                                  GHashTable        *items,
                                   NautilusDirectory *directory)
 {
     g_autoptr (NautilusFile) parent = nautilus_directory_get_corresponding_file (directory);
     GListStore *dir_store = get_directory_store (self, parent);
-    guint new_start, current_start;
-    guint n_items_in_range = 0;
-    g_autoptr (GtkBitset) positions = gtk_bitset_new_empty ();
-    GtkBitsetIter position_iter;
+    guint range_start, n_items_in_range = 0;
 
-    for (GList *l = items; l != NULL; l = l->next)
+    /* Remove contiguous item ranges to minimize ::items-changed emissions.
+     * Remove after passing the range to not impact the index. */
+    for (gint i = g_list_model_get_n_items (G_LIST_MODEL (dir_store)) - 1;
+         i >= 0 && g_hash_table_size (items) > 0; i--)
     {
-        NautilusViewItem *item = l->data;
-        NautilusFile *file = nautilus_view_item_get_file (item);
-        guint i;
+        g_autoptr (NautilusViewItem) item = g_list_model_get_item (G_LIST_MODEL (dir_store), i);
+        NautilusFile *file = NULL;
 
-        if (!g_list_store_find (dir_store, item, &i))
+        if (!g_hash_table_steal_extended (items, item, NULL, (gpointer *) &file))
         {
-            g_autofree char *uri = nautilus_file_get_uri (file);
+            if (n_items_in_range > 0)
+            {
+                g_list_store_splice (dir_store, range_start, n_items_in_range, NULL, 0);
+                n_items_in_range = 0;
+            }
 
-            g_warning ("Failed to remove item %s", uri);
             continue;
         }
 
-        gtk_bitset_add (positions, i);
         g_hash_table_remove (self->map_files_to_model, file);
         if (nautilus_file_is_directory (file))
         {
             g_hash_table_remove (self->directory_reverse_map, file);
         }
+
+        /* The previous item is contiguous, keep growing the range. */
+        n_items_in_range++;
+        range_start = i;
     }
 
-    /* Remove contiguous item ranges to minimize ::items-changed emissions.
-     * Remove starting from the end, not to impact the index */
-    gtk_bitset_iter_init_last (&position_iter, positions, &new_start);
-    while (gtk_bitset_iter_is_valid (&position_iter))
+    if (n_items_in_range > 0)
     {
-        /* The start of the range is a moving target that gets decremented as we
-         * move up the list. */
-        current_start = new_start;
-        n_items_in_range++;
+        /* Flush the leftover range. */
+        g_list_store_splice (dir_store, range_start, n_items_in_range, NULL, 0);
+    }
 
-        if (gtk_bitset_iter_previous (&position_iter, &new_start) &&
-            new_start == current_start - 1)
-        {
-            /* The previous item is contiguous, keep growing the range. */
-            continue;
-        }
-
-        /* Flush full-grown range. */
-        g_list_store_splice (dir_store, current_start, n_items_in_range, 0, 0);
-        n_items_in_range = 0;
+    if (g_hash_table_size (items) > 0)
+    {
+        g_warning ("Failed to remove %u item(s)", g_hash_table_size (items));
     }
 }
 
