@@ -10,11 +10,17 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (NautilusViewModel, nautilus_view_model, NAUTILUS, VIEW_MODEL, GObject)
 
-NautilusViewModel * nautilus_view_model_new (void);
+NautilusViewModel * nautilus_view_model_new (gboolean single_selection);
 
+GtkFilter *nautilus_view_model_get_filter (NautilusViewModel *self);
+void nautilus_view_model_set_filter (NautilusViewModel *self,
+                                     GtkFilter         *filter);
+gboolean nautilus_view_model_get_single_selection (NautilusViewModel *self);
 GtkSorter *nautilus_view_model_get_sorter (NautilusViewModel *self);
 void nautilus_view_model_set_sorter (NautilusViewModel *self,
                                      GtkSorter         *sorter);
+void nautilus_view_model_set_section_sorter (NautilusViewModel *self,
+                                             GtkSorter         *section_sorter);
 void nautilus_view_model_sort (NautilusViewModel *self);
 NautilusViewItem * nautilus_view_model_get_item_for_file (NautilusViewModel *self,
                                                           NautilusFile      *file);
@@ -22,7 +28,7 @@ GList * nautilus_view_model_get_sorted_items_for_files (NautilusViewModel *self,
                                                         GList             *files);
 /* Don't use inside a loop, use nautilus_view_model_remove_all_items instead. */
 void nautilus_view_model_remove_items (NautilusViewModel     *self,
-                                       GList                 *items,
+                                       GHashTable            *items,
                                        NautilusDirectory     *directory);
 void nautilus_view_model_remove_all_items (NautilusViewModel *self);
 /* Don't use inside a loop, use nautilus_view_model_add_items instead. */
