@@ -23,13 +23,7 @@
 
 #include "nautilus-query.h"
 #include "nautilus-toolbar-menu-sections.h"
-
-/* Keep values in sync with the org.gnome.nautilus.FolderView schema enums: */
-#define NAUTILUS_VIEW_LIST_ID            1
-#define NAUTILUS_VIEW_GRID_ID            2
-/* Special ids, not used by GSettings schemas: */
-#define NAUTILUS_VIEW_INVALID_ID         0
-#define NAUTILUS_VIEW_OTHER_LOCATIONS_ID 3
+#include "nautilus-view-info.h"
 
 G_BEGIN_DECLS
 
