@@ -194,7 +194,7 @@ action_go_home (GSimpleAction *action,
     window = NAUTILUS_WINDOW (user_data);
     home = g_file_new_for_path (g_get_home_dir ());
 
-    nautilus_window_open_location_full (window, home, 0, NULL, NULL, NULL);
+    nautilus_window_open_location_full (window, home, 0, NULL, NULL);
 
     g_object_unref (home);
 }
@@ -437,8 +437,7 @@ nautilus_window_open_location_full (NautilusWindow     *window,
                                     GFile              *location,
                                     NautilusOpenFlags   flags,
                                     GList              *selection,
-                                    NautilusWindowSlot *target_slot,
-                                    const char         *startup_id)
+                                    NautilusWindowSlot *target_slot)
 {
     NautilusWindowSlot *active_slot;
 
@@ -460,10 +459,6 @@ nautilus_window_open_location_full (NautilusWindow     *window,
      * opposite, since it's the most usual use case */
     if (!(flags & NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE))
     {
-        if (startup_id)
-        {
-            gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
-        }
         gtk_window_present (GTK_WINDOW (window));
         nautilus_window_set_active_slot (window, target_slot);
     }
@@ -524,7 +519,7 @@ nautilus_window_new_tab (NautilusWindow *window)
 
         nautilus_window_open_location_full (window, location,
                                             NAUTILUS_OPEN_FLAG_NEW_TAB,
-                                            NULL, NULL, NULL);
+                                            NULL, NULL);
         g_object_unref (location);
     }
 }
@@ -870,11 +865,6 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
             GFile *location;
 
             active_slot = nautilus_window_get_active_slot (window);
-            if (active_slot == NULL)
-            {
-                return;
-            }
-
             location = nautilus_window_slot_get_location (active_slot);
             /* Don't pop up a notification if the focus is not in the this
              * window. This is an easy way to know from which window was the
@@ -1720,6 +1710,7 @@ nautilus_window_new (void)
 void
 nautilus_window_show_about_dialog (NautilusWindow *window)
 {
+    AdwDialog *dialog;
     g_autofree gchar *module_names = nautilus_module_get_installed_module_names ();
     g_autofree gchar *debug_info = NULL;
 
@@ -1752,26 +1743,20 @@ nautilus_window_show_about_dialog (NautilusWindow *window)
                                   "NAUTILUS_DISABLE_PLUGINS=TRUE nautilus", NULL);
     }
 
-    adw_show_about_dialog (window ? GTK_WIDGET (window) : NULL,
-                           "application-name", _("Files"),
-                           "application-icon", APPLICATION_ID,
-                           "developer-name", _("The GNOME Project"),
-                           "version", VERSION,
-                           "website", "https://apps.gnome.org/Nautilus/",
-                           "issue-url", "https://gitlab.gnome.org/GNOME/nautilus/-/issues",
-                           "support-url", "https://discourse.gnome.org/tag/nautilus",
-                           "debug-info", debug_info,
-                           "copyright", "© 1999 The Files Authors",
-                           "license-type", GTK_LICENSE_GPL_3_0,
-                           "designers", designers,
-                           "developers", developers,
-                           "documenters", documenters,
-                           /* Translators should localize the following string
-                            * which will be displayed at the bottom of the about
-                            * box to give credit to the translator(s).
-                            */
-                           "translator-credits", _("translator-credits"),
-                           NULL);
+    dialog = adw_about_dialog_new_from_appdata ("/org/gnome/nautilus/appdata", NULL);
+
+    adw_about_dialog_set_version (ADW_ABOUT_DIALOG (dialog), VERSION);
+    adw_about_dialog_set_debug_info (ADW_ABOUT_DIALOG (dialog), debug_info);
+    adw_about_dialog_set_copyright (ADW_ABOUT_DIALOG (dialog), "© 1999 The Files Authors");
+    adw_about_dialog_set_developers (ADW_ABOUT_DIALOG (dialog), developers);
+    adw_about_dialog_set_designers (ADW_ABOUT_DIALOG (dialog), designers);
+    adw_about_dialog_set_documenters (ADW_ABOUT_DIALOG (dialog), documenters);
+    adw_about_dialog_set_support_url (ADW_ABOUT_DIALOG (dialog), "https://discourse.gnome.org/tag/nautilus");
+    /* Translators should localize the following string which will be displayed at the bottom of
+     * the about box to give credit to the translator(s). */
+    adw_about_dialog_set_translator_credits (ADW_ABOUT_DIALOG (dialog), _("translator-credits"));
+
+    adw_dialog_present (dialog, window ? GTK_WIDGET (window) : NULL);
 }
 
 void
